# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 HL eurhat@hotmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import random
import os

from gi.repository import GObject
from gi.repository import Notify
from gi.repository import AppIndicator3 as AppIndicator
from gi.repository import Gio

import gettext
from gettext import gettext as _
gettext.textdomain('notifis')

from gi.repository import Gtk # pylint: disable=E0611
import logging
logger = logging.getLogger('notifis')

from notifis_lib import Window
from notifis.AboutNotifisDialog import AboutNotifisDialog
#from notifis.PreferencesNotifisDialog import PreferencesNotifisDialog

# See notifis_lib.Window.py for more details about how this class works
class NotifisWindow(Window):  
    __gtype_name__ = "NotifisWindow"

    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(NotifisWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutNotifisDialog
        #self.PreferencesDialog = PreferencesNotifisDialog
    
        # Code for other initialization actions should be added here.
        self.connect("delete-event", self.minimize) # for minize to systray; thank jezra (www.jezra.net)
        self.btnOk = builder.get_object("btnOk")
        self.btnNTime = builder.get_object("btnNTime")
        self.timerID = -1
        self.btnFile = builder.get_object("btnFile")
        self.chkShowAtStartup = builder.get_object("chkShowAtStartup")
        self.settings = Gio.Settings("net.launchpad.notifis")
            
        # the app indicator        
        self.indMenu = Gtk.Menu()
        self.mnuClose = Gtk.MenuItem("Close")
        self.mnuOpen = Gtk.MenuItem("Settings")
        self.mnuNext = Gtk.MenuItem("Next")
        self.mnuClose.connect("activate", self.app_close)
        self.mnuOpen.connect("activate", self.app_open)
        self.mnuNext.connect("activate", self.do_bubble)
        self.indMenu.append(self.mnuNext)
        self.indMenu.append(self.mnuOpen)
        self.indMenu.append(self.mnuClose)        
        self.mnuNext.show()        
        self.mnuClose.show()
        path = os.path.join(os.path.dirname(__file__), '../share/notifis/media/notifis.svg')
        print path
        self.appIndicator = AppIndicator.Indicator.new("Notifis", path, AppIndicator.IndicatorCategory.APPLICATION_STATUS)
        self.appIndicator.set_status(AppIndicator.IndicatorStatus.ACTIVE)
        self.appIndicator.set_menu(self.indMenu)
        Notify.init("Notifis")
        
        # read settings  
        showup = self.settings.get_boolean("show-at-startup")     
        self.chkShowAtStartup.set_active(showup) 
        timeinterval = self.settings.get_int("time-interval")
        self.btnNTime.set_value(timeinterval)
        filepath = self.settings.get_string("file-path")
        if filepath.strip():
            self.btnFile.set_filename(filepath)
        else:
            defpath = os.path.join(os.path.dirname(__file__), '../share/notifis/notifis_default.txt')
            self.btnFile.set_filename(defpath)
            self.settings.set_string("file-path", defpath)
            filepath = defpath            

        # read file
        if filepath.strip():
            try:
                f = open(filepath,"r")
            except IOError:
                dlg = Gtk.MessageDialog(self, Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE, "Fail to read file !")  
                dlg.run()
                dlg.destroy()    
            else:            
                self.lines = f.readlines()
                f.close()
                self.lines.insert(0,"\n")
         
            if timeinterval:                       
                self.timerID = GObject.timeout_add(timeinterval*60*1000, self.do_bubble, True)
            self.do_bubble(False)  
     
    # show notification
    def do_bubble(self, repeat=True): 
        # select a random line, move back to nearest blank line                    
        i = random.randint(0,len(self.lines)-1)
        while self.lines[i].strip():
                i -= 1
        # go to the start if meet last line       
        i += 1
        if i==len(self.lines):
            i=1
            while i<len(self.lines) and not self.lines[i].strip():
                i += 1
        
        # construct message and show                
        message = ""
        while i<len(self.lines) and self.lines[i].strip():        
            message += self.lines[i]
            i += 1 
        n=Notify.Notification.new("(^ᴥ^)",message,"")
        n.show() 
        return repeat 

    def on_btnFile_file_set(self, button):
        f = open(self.btnFile.get_filename(),"r")
        self.lines = f.readlines()
        f.close()
        self.lines.insert(0,"\n") 
        # check input errors
    
    def on_btnOk_clicked(self, button, data=None):
        filepath =  self.btnFile.get_filename()  
        oldpath = self.settings.get_string("file-path")                        
        if filepath and (filepath != oldpath):            
            try:
                f = open(filepath,"r")
            except IOError:
                dlg = Gtk.MessageDialog(self, Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE, "Fail to read file !")  
                dlg.run()
                dlg.destroy()     
            else:
                self.settings.set_string("file-path", filepath)            
                self.lines = f.readlines()
                f.close()
                self.lines.insert(0,"\n")            
        else:
            self.btnFile.set_filename(oldpath)
        self.settings.set_boolean("show-at-startup", self.chkShowAtStartup.get_active())
        timeinterval = self.btnNTime.get_value_as_int()        
        self.settings.set_int("time-interval", timeinterval)
        if (self.timerID != -1):        
            GObject.source_remove(self.timerID)
            self.timerID = -1
        if timeinterval:    
            self.timerID = GObject.timeout_add(timeinterval*60*1000, self.do_bubble, True)        
        self.minimize(self, button)
    
    def app_close(self, widget):
        Gtk.main_quit()

    def app_open(self, widget):        
        self.show_all()
        self.mnuOpen.hide()
    
    def minimize(self, widget, evt):
        self.hide_on_delete()
        self.mnuOpen.show()
        return True
   
   

